﻿using ProjectSellPC.Forms.Clients;
using ProjectSellPC.Repos;
using Unity;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ProjectSellPC.Forms
{
    public partial class ClientForm : Form
    {
        private readonly IUnityContainer _container;
        private readonly IClientRepository _repository;

        public ClientForm(IUnityContainer unityContainer, IClientRepository clientRepository)
        {
            InitializeComponent();

            _container = unityContainer ?? throw new ArgumentNullException(nameof(unityContainer));
            _repository = clientRepository ?? throw new ArgumentNullException(nameof(_repository));
        }

        private void ClientForm_Load(object sender, EventArgs e)
        {
            try
            {
                LoadList();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при загрузке", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        private void LoadList() => productsDataGridView.DataSource = _repository.ReadAll();

        private bool TryGetIdentifierFromSelectedRow(out int id)
        {
            id = 0;
            if (productsDataGridView.SelectedRows.Count < 1)
            {
                MessageBox.Show("Нет выбранной записи", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }

            id = Convert.ToInt32(productsDataGridView.SelectedRows[0].Cells["Id"].Value);
            return true;
        }

        private void addButton_Click(object sender, EventArgs e)
        {
            try
            {
                _container.Resolve<ClientFormSettings>().ShowDialog();
                LoadList();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при добавлении", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void editButton_Click(object sender, EventArgs e)
        {
            if (!TryGetIdentifierFromSelectedRow(out var findId))
            {
                return;
            }

            try
            {
                var form = _container.Resolve<ClientFormSettings>();
                form.Id = findId;
                form.ShowDialog();
                LoadList();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при изменении", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void deleteButton_Click(object sender, EventArgs e)
        {
            if (!TryGetIdentifierFromSelectedRow(out var findId))
            {
                return;
            }

            if (MessageBox.Show("Удалить запись?", "Удаление", MessageBoxButtons.YesNo) != DialogResult.Yes)
            {
                return;
            }

            try
            {
                _repository.Delete(findId);
                LoadList();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при удалении", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
    }
}
