﻿using ProjectSellPC.Entites;
using Dapper;
using Microsoft.Extensions.Logging;
using Npgsql;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;

namespace ProjectSellPC.Repos.Impements
{
    public class ChequeRepo : IChequeRepository
    {
        private readonly IConnectionString _connectionString;
        private readonly ILogger<ChequeRepo> _logger;
        private readonly IClientRepository _clientRepository;

        public ChequeRepo(IConnectionString connectionString, ILoggerFactory loggerFactory, IClientRepository clientRepository)
        {
            _connectionString = connectionString ?? throw new ArgumentNullException(nameof(connectionString));
            _logger = loggerFactory.CreateLogger<ChequeRepo>();
            _clientRepository = clientRepository ?? throw new ArgumentNullException(nameof(clientRepository));
        }

        private IDbConnection CreateConnection() => new NpgsqlConnection(_connectionString.ConnectionString);
        public void Create(Cheque Cheque)
        {
            _logger.LogInformation("Создание чека");
            using (var connection = CreateConnection())
            {
                connection.Open();
                using (var transaction = connection.BeginTransaction())
                {
                    try
                    {
                        var chequeSql = "INSERT INTO \"cheque\" (\"clientid\", \"purchasedate\") VALUES (@ClientId, @PurchaseDate) RETURNING \"id\"";
                        Cheque.Id = connection.ExecuteScalar<int>(chequeSql, new
                        {
                            ClientId = Cheque.Client.Id,
                            PurchaseDate = Cheque.PurchaseDate
                        }, transaction);

                        var productSql = "INSERT INTO \"productincheque\" (\"chequeid\", \"productid\", \"count\") VALUES (@ChequeId, @ProductID, @Count)";
                        foreach (var productInCheque in Cheque.Products)
                        {
                            connection.Execute(productSql, new
                            {
                                ChequeId = Cheque.Id,
                                ProductID = productInCheque.ProductID,
                                Count = productInCheque.Count
                            }, transaction);
                        }

                        transaction.Commit();
                    }
                    catch (Exception ex)
                    {
                        transaction.Rollback();
                        _logger.LogError(ex, "Ошибка при создании чека");
                        throw;
                    }
                }
            }
        }

        public Cheque Read(int id)
        {
            _logger.LogInformation("Чтение чека по ID: {id}", id);
            using (var connection = CreateConnection())
            {
                try
                {
                    var ChequeSql = "SELECT * FROM \"cheque\" WHERE \"id\" = @Id";
                    var Cheque = connection.QuerySingleOrDefault<Cheque>(ChequeSql, new { Id = id });

                    if (Cheque == null) return null;

                    Cheque.Client = _clientRepository.Read(Cheque.Client.Id);

                    var productSql = "SELECT * FROM \"productincheque\" WHERE \"chequeid\" = @ChequeId";
                    Cheque.Products = connection.Query<ProductInCheque>(productSql, new { ChequeId = Cheque.Id }).ToList();

                    return Cheque;
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка при чтении чека");
                    throw;
                }
            }
        }

        public IEnumerable<Cheque> ReadAll()
        {
            _logger.LogInformation("Чтение всех чеков");
            using (var connection = CreateConnection())
            {
                try
                {
                    var ChequeSql = "SELECT * FROM \"cheque\"";
                    var Cheques = connection.Query<Cheque>(ChequeSql).ToList();

                    foreach (var Cheque in Cheques)
                    {
                        Cheque.Client = _clientRepository.Read(Cheque.ClientId);

                        var productSql = "SELECT * FROM \"productincheque\" WHERE \"chequeid\" = @ChequeId";
                        Cheque.Products = connection.Query<ProductInCheque>(productSql, new { ChequeId = Cheque.Id }).ToList();
                    }

                    return Cheques;
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка при чтении всех чеков");
                    throw;
                }
            }
        }
    }
}
