﻿using ProjectSellPC.Entites;
using ProjectSellPC.Entites.Enums;
using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using System;
using System.Collections.Generic;
using System.Data;

namespace ProjectSellPC.Repos.Impements
{
    public class ClientRepo : IClientRepository
    {
        private readonly IConnectionString _connectionString;
        private readonly ILogger<ClientRepo> _logger;

        public ClientRepo(IConnectionString connectionString, ILoggerFactory loggerFactory)
        {
            _connectionString = connectionString;
            _logger = loggerFactory.CreateLogger<ClientRepo>();
        }

        private IDbConnection CreateConnection() => new NpgsqlConnection(_connectionString.ConnectionString);
        public void Create(Client client)
        {
            _logger.LogInformation("Добавление клиента");
            _logger.LogDebug("Клиент: {json}", JsonConvert.SerializeObject(client));

            try
            {
                using (var connection = CreateConnection())
                {
                    var sql = "INSERT INTO \"client\" (\"name\", \"phonenumber\", \"clienttype\") " +
                              "VALUES (@Name, @PhoneNumber, @ClientType)";

                    connection.Execute(sql, new
                    {
                        Name = client.Name,
                        PhoneNumber = client.PhoneNumber,
                        ClientType = (int)client.ClientType
                    });
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при добавлении клиента");
                throw;
            }
        }

        public void Delete(int id)
        {
            _logger.LogInformation("Удаление клиента");
            _logger.LogDebug("Клиент ID: {id}", id);

            try
            {
                using (var connection = CreateConnection())
                {
                    var sql = "DELETE FROM \"client\" WHERE \"id\" = @Id";
                    connection.Execute(sql, new { Id = id });
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при удалении клиента");
                throw;
            }
        }

        public Client Read(int id)
        {
            _logger.LogInformation("Получение клиента по ID");
            _logger.LogDebug("Клиент ID: {id}", id);

            try
            {
                using (var connection = CreateConnection())
                {
                    var sql = "SELECT * FROM \"client\" WHERE \"id\" = @Id";
                    return connection.QuerySingleOrDefault<Client>(sql, new { Id = id });
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при получении клиента");
                throw;
            }
        }

        public IEnumerable<Client> ReadAll()
        {
            _logger.LogInformation("Получение всех клиентов");

            try
            {
                using (var connection = CreateConnection())
                {
                    var sql = "SELECT * FROM \"client\"";
                    return connection.Query<Client>(sql);
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при получении списка клиентов");
                throw;
            }
        }

        public Client Update(Client client)
        {
            _logger.LogInformation("Обновление информации о клиенте");
            _logger.LogDebug("Клиент: {json}", JsonConvert.SerializeObject(client));

            try
            {
                using (var connection = CreateConnection())
                {
                    var sql = "UPDATE \"client\" SET \"name\" = @Name, \"phonenumber\" = @PhoneNumber, \"clienttype\" = @ClientType " +
                              "WHERE \"id\" = @Id";

                    connection.Execute(sql, new
                    {
                        Id = client.Id,
                        Name = client.Name,
                        PhoneNumber = client.PhoneNumber,
                        ClientType = (int)client.ClientType
                    });

                    return client;
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при обновлении информации о клиенте");
                throw;
            }
        }
    }
}
